# !/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Read File Content """


from baseblock import FileIO
from baseblock import Enforcer
from baseblock import BaseObject


class ReadFileContent(BaseObject):
    """ Read File Content """

    def __init__(self):
        """ Change Log

        Created:
            5-Sept-2022
            craigtrim@gmail.com
        """
        BaseObject.__init__(self, __name__)

    def process(self,
                input_files: list) -> dict:

        if self.isEnabledForDebug:
            Enforcer.is_list_of_str(input_files)

        [FileIO.exists_or_error(input_file)
         for input_file in input_files]

        d = {}
        for input_file in input_files:
            d[input_file] = FileIO.read_lines(input_file)

        return d



# !/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Metrics to Find CC (Cyclomatic Complexity) metrics """


from baseblock import BaseObject


class MetricCCCounter(BaseObject):
    """ Metrics to Find CC (Cyclomatic Complexity) metrics """

    def __init__(self):
        """ Change Log

        Created:
            5-Sept-2022
            craigtrim@gmail.com
        """
        BaseObject.__init__(self, __name__)

    def process(self,
                input_lines: list) -> dict:

        input_lines = [x.strip() for x in input_lines if x and len(x)]
        total_definitions = len([x for x in input_lines
                                 if x.startswith('def ')])

        calls = 0
        for input_line in input_lines:
            if 'self.' in input_line or 'cls.' in input_line:
                calls += 1

        return calls + total_definitions



# !/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Execute Orchestrator with Parameters """


from baseblock import FileIO

from micro_metrics.bp import Orchestrator


def run(input_directory: str,
        output_file: str) -> str:
    metrics = Orchestrator().run(input_directory)
    FileIO.write_json(metrics, output_file)


def main(input_directory):
    run(input_directory)


if __name__ == "__main__":
    import plac

    plac.call(main)



from .metric_loc_counter import MetricLocCounter
from .metric_tloc_counter import MetricTLocCounter
from .metric_fp_counter import MetricFpCounter
from .metric_import_counter import MetricImportCounter
from .metric_cc_counter import MetricCCCounter



# !/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Heuristic Computation of Complexity """


from baseblock import BaseObject


class ComputeComplexity(BaseObject):
    """ Heuristic Computation of Complexity """

    def __init__(self):
        """ Change Log

        Created:
            5-Sept-2022
            craigtrim@gmail.com
        """
        BaseObject.__init__(self, __name__)

    def process(self,
                d_sum: dict) -> int:

        def loc() -> float:
            if 'LOC' in d_sum and d_sum['LOC']:
                return d_sum['LOC'] * 0.25
            return 0

        def tloc() -> float:
            if 'TLOC' in d_sum and d_sum['TLOC']:
                return d_sum['TLOC'] * 0.05
            return 0

        def ccs() -> float:
            if 'CC' in d_sum and d_sum['CC']:
                return d_sum['CC'] * 0.75
            return 0

        def fps() -> float:
            if 'FP' in d_sum and d_sum['FP']:
                return d_sum['FP'] * 0.50
            return 0

        def ipynb() -> float:
            if 'IPYNB' in d_sum and d_sum['IPYNB']:
                return d_sum['IPYNB'] * 2
            return 0

        def imports() -> float:
            if 'Imports' in d_sum and d_sum['Imports']:
                return d_sum['Imports'] * 0.15
            return 0

        def dmo() -> float:
            if 'DMO' in d_sum and d_sum['DMO']:
                return d_sum['DMO'] * 0.80
            return 0

        def svc() -> float:
            if 'SVC' in d_sum and d_sum['SVC']:
                return d_sum['SVC'] * 1.00
            return 0

        def bp() -> float:
            if 'BP' in d_sum and d_sum['BP']:
                return d_sum['BP'] * 1.10
            return 0

        def dto() -> float:
            if 'DTO' in d_sum and d_sum['DTO']:
                return d_sum['DTO'] * 0.60
            return 0

        def tests() -> float:
            if 'TESTS' in d_sum and d_sum['TESTS']:
                return d_sum['TESTS'] * 1.50
            return 0

        total = loc() + tloc() + ccs() + fps() + imports() + \
            dmo() + svc() + bp() + dto() + tests() + ipynb()

        return int(round(total, 0))



# !/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Metrics to Find LoC (Lines-of-Code) metrics """


from baseblock import BaseObject


class MetricLocCounter(BaseObject):
    """ Metrics to Find LoC (Lines-of-Code) metrics """

    def __init__(self):
        """ Change Log

        Created:
            5-Sept-2022
            craigtrim@gmail.com
        """
        BaseObject.__init__(self, __name__)

    def process(self,
                input_lines: list) -> int:

        input_lines = [x for x in input_lines if x and len(x)]

        return len(input_lines)



# !/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Metrics to Find Structural Metrics for the Microservice """


from baseblock import FileIO
from baseblock import BaseObject


class FindFolderMetrics(BaseObject):
    """ Metrics to Find Structural Metrics for the Microservice """

    def __init__(self):
        """ Change Log

        Created:
            5-Sept-2022
            craigtrim@gmail.com
        """
        BaseObject.__init__(self, __name__)

    def process(self,
                input_directory: str) -> dict:

        d_results = {}

        d_folders = FileIO.load_all_folders(
            input_directory,
            exclude=['.git', '__pycache__', '.pytest_cache'])

        for folder_name in d_folders:
            if folder_name in ['bp', 'svc', 'dmo', 'dto', 'tests']:
                d_results[folder_name] = len(d_folders[folder_name])

        return {k.upper(): d_results[k] for k in d_results}



# !/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Metrics to Find LoC (Lines-of-Code) metrics """


from baseblock import Enforcer
from baseblock import BaseObject

from micro_metrics.dmo import MetricFpCounter
from micro_metrics.dmo import MetricImportCounter
from micro_metrics.dmo import MetricLocCounter
from micro_metrics.dmo import MetricTLocCounter
from micro_metrics.dmo import MetricCCCounter


class FindFileMetrics(BaseObject):
    """ Metrics to Find LoC (Lines-of-Code) metrics """

    def __init__(self):
        """ Change Log

        Created:
            5-Sept-2022
            craigtrim@gmail.com
        """
        BaseObject.__init__(self, __name__)
        self._loc = MetricLocCounter().process
        self._tloc = MetricTLocCounter().process
        self._fps = MetricFpCounter().process
        self._ccs = MetricCCCounter().process
        self._imports = MetricImportCounter().process

    def process(self,
                d_input_files: dict) -> dict:

        if self.isEnabledForDebug:
            Enforcer.is_dict(d_input_files)

        results = []
        for file_name in d_input_files:
            input_lines = d_input_files[file_name]

            results.append({
                "FileName": file_name,
                "LOC": self._loc(input_lines),
                "TLOC": self._tloc(input_lines),
                "FP": self._fps(input_lines),
                "CC": self._ccs(input_lines),
                "Imports": self._imports(input_lines)
            })

        return results
